<?php

namespace NoccyLabs\JitEnabler;

class Jit
{

    const OPTIMIZE_NONE = 0;
    const OPTIMIZE_MINIMAL = 1;
    const OPTIMIZE_SELECTIVE_INLINING = 2;
    const OPTIMIZE_TYPE = 3;
    const OPTIMIZE_TYPE_TREE = 4;
    const OPTIMIZE_TYPE_INNER = 5;

    const JIT_ON_LOAD = 0;
    const JIT_ON_EXEC = 1;
    const JIT_TWO_PASS = 2;
    const JIT_ON_THE_FLY = 3;
    const JIT_DOCCOMMENT = 4;
    const JIT_TRACING = 5;

    const REGISTER_NOALLOC = 0;
    const REGISTER_LOCAL = 1;
    const REGISTER_GLOBAL = 2;

    const CPU_NONE = 0;
    const CPU_AVX = 1;

    public static string $bufferSize = "100M";

    public static int $optimizationLevel = self::OPTIMIZE_TYPE_INNER;

    public static int $jitTrigger = self::JIT_TRACING;

    public static int $registerAllocation = self::REGISTER_GLOBAL;

    public static int $cpuFeature = self::CPU_AVX;



    public static function enable()
    {

        if (self::checkBlacklistedExtensions()) return;

        $selfArgs = $GLOBALS['argv'];
        $selfCommand = array_shift($selfArgs);

        // Check to see if activation failed
        if (opcache_get_status() === false && intval(ini_get('opcache.jit_buffer_size')) > 0) {
            //echo "jit failed to enable\n";
            if (!defined("JIT_ACTIVE"))
                define("JIT_ACTIVE", false);
            return;
        } elseif (opcache_get_status() !== false) {
            //echo "jit enabled\n";
            if (!defined("JIT_ACTIVE"))
                define("JIT_ACTIVE", true);
            return;
        }
        //echo "enabling jit\n";

        $jitArgs = [
            sprintf('-dopcache.enable_cli=%d', 1),
            sprintf('-dopcache.jit_buffer_size=%s', self::$bufferSize),
            sprintf('-dopcache.jit=%d%d%d%d', self::$cpuFeature, self::$registerAllocation, self::$jitTrigger, self::$optimizationLevel),
        ];

        if (preg_match('<^php[-]([0-9.]+)$>', basename($selfCommand))) {
            $newCommand = [ $selfCommand, ...$jitArgs, '--', ...$selfArgs ];
        } else {
            $newCommand = [ 'php', ...$jitArgs, $selfCommand, ...$selfArgs ];
        }

        $newCommand = join(" ", [ escapeshellcmd(array_shift($newCommand)), ...array_map('escapeshellarg', $newCommand) ]);

        //var_dump($newCommand);
        passthru($newCommand, $res);
        exit($res);

    }

    public static function checkBlacklistedExtensions(): bool
    {
        // Check the blacklist of extensions that prevent jitting
        $blacklist = [ 'xdebug' ];
        foreach ($blacklist as $extension) {
            if (extension_loaded($extension)) {
                return true;
            }
        }
        return false;
    }


}
